#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

static int ouraddrfor(struct in_addr *us, struct in_addr *them)
{
	int s;
	struct sockaddr_in sin;
        socklen_t slen;
	
        s = socket(PF_INET, SOCK_DGRAM, 0);
        if (s == -1) {
                perror("socket");
                return -1;
        }
	sin.sin_addr = *them;
        sin.sin_family = AF_INET;
        sin.sin_port = 5060;
        if (connect(s, (struct sockaddr *)&sin, sizeof(sin))) {
                perror("connect");
                return -1;
        }
        slen = sizeof(sin);
        if (getsockname(s, (struct sockaddr *)&sin, &slen)) {
                perror("getsockname");
                return -1;
        }
        close(s);
	*us = sin.sin_addr;
	return 0;
}


int main(int argc, char *argv[])
{
	struct in_addr us, them;
        struct hostent *h;
	struct timeval tv1, tv2;
	int r, fd;
	char buf[8192];

        if (argc < 2) {
                fprintf(stderr, "Usage: ipaddrtest <remotehost>\n");
                return 1;
        }
        if (!inet_aton(argv[1], &them)) {
                h = gethostbyname(argv[1]);
                if (!h) {
                        fprintf(stderr, "Cannot resolve \"%s\"\n", argv[1]);
                        return 1;
                }
                if (h->h_addrtype != AF_INET) {
                        fprintf(stderr, "%s: wrong address type\n", h->h_name);
                        return 1;
                }
                them = *(struct in_addr *)h->h_addr_list[0];
                endhostent();
        }
	gettimeofday(&tv1, NULL);
	r = ouraddrfor(&us, &them);
	gettimeofday(&tv2, NULL);
	if (!r) {
		printf("our address: %s, time %luus\n",
		       inet_ntoa(us),
		       (tv2.tv_sec - tv1.tv_sec) * 1000000 + tv2.tv_usec - tv1.tv_usec);
	}
	/* time */
	gettimeofday(&tv1, NULL);
	fd = open("/proc/net/route", O_RDONLY);
	if (fd >= 0) {
		read(fd, buf, sizeof(buf));
		close(fd);
	} else
		perror("open: /proc/net/route");
	gettimeofday(&tv2, NULL);
	printf("time to read /proc/net/route %luus\n",
	       (tv2.tv_sec - tv1.tv_sec) * 1000000 + tv2.tv_usec - tv1.tv_usec);	
        return -r;
}
